@@ -126,7 +126,6 @@ static gboolean grl_bookmarks_source_notify_change_stop (GrlSource *source,
                             GrlPlugin *plugin,
                             GList *configs)
  {
-   GParamSpec *spec;
    GRL_LOG_DOMAIN_INIT (bookmarks_log_domain, "bookmarks");
 
    GRL_DEBUG ("grl_bookmarks_plugin_init");
@@ -135,25 +134,6 @@ static gboolean grl_bookmarks_source_notify_change_stop (GrlSource *source,
    bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
    bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 
-   spec = g_param_spec_boxed ("bookmark-date",
-                              "Bookmark date",
-                              "When the media was bookmarked",
-                              G_TYPE_DATE_TIME,
-                              G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE),
-   GRL_BOOKMARKS_KEY_BOOKMARK_TIME =
-       grl_registry_register_metadata_key (registry, spec, NULL);
-   /* If key was not registered, could be that it is already registered. If so,
-      check if type is the expected one, and reuse it */
-   if (GRL_BOOKMARKS_KEY_BOOKMARK_TIME == GRL_METADATA_KEY_INVALID) {
-     g_param_spec_unref (spec);
-     GRL_BOOKMARKS_KEY_BOOKMARK_TIME =
-         grl_registry_lookup_metadata_key (registry, "bookmark-date");
-     if (grl_metadata_key_get_type (GRL_BOOKMARKS_KEY_BOOKMARK_TIME)
-         != G_TYPE_DATE_TIME) {
-       GRL_BOOKMARKS_KEY_BOOKMARK_TIME = GRL_METADATA_KEY_INVALID;
-     }
-   }
-
    GrlBookmarksSource *source = grl_bookmarks_source_new ();
    grl_registry_register_source (registry,
                                  plugin,
@@ -162,9 +142,36 @@ static gboolean grl_bookmarks_source_notify_change_stop (GrlSource *source,
    return TRUE;
  }
 
- GRL_PLUGIN_REGISTER (grl_bookmarks_plugin_init,
-                      NULL,
-                      PLUGIN_ID);
+static void
+grl_bookmarks_plugin_register_keys (GrlRegistry *registry,
+                                    GrlPlugin   *plugin)
+{
+  GParamSpec *spec;
+
+  spec = g_param_spec_boxed ("bookmark-date",
+                             "Bookmark date",
+                             "When the media was bookmarked",
+                             G_TYPE_DATE_TIME,
+                             G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE),
+  GRL_BOOKMARKS_KEY_BOOKMARK_TIME =
+      grl_registry_register_metadata_key (registry, spec, NULL);
+  /* If key was not registered, could be that it is already registered. If so,
+     check if type is the expected one, and reuse it */
+  if (GRL_BOOKMARKS_KEY_BOOKMARK_TIME == GRL_METADATA_KEY_INVALID) {
+    g_param_spec_unref (spec);
+    GRL_BOOKMARKS_KEY_BOOKMARK_TIME =
+        grl_registry_lookup_metadata_key (registry, "bookmark-date");
+    if (grl_metadata_key_get_type (GRL_BOOKMARKS_KEY_BOOKMARK_TIME)
+        != G_TYPE_DATE_TIME) {
+      GRL_BOOKMARKS_KEY_BOOKMARK_TIME = GRL_METADATA_KEY_INVALID;
+    }
+  }
+}
+
+ GRL_PLUGIN_REGISTER_FULL (grl_bookmarks_plugin_init,
+                           NULL,
+                           grl_bookmarks_plugin_register_keys,
+                           PLUGIN_ID);
 
  /* ================== Bookmarks GObject ================ */
 
@@ -472,7 +479,6 @@ find_cb (GObject      *object,
     goto out;
   }
 
-  idx = os->skip;
   num_left = MIN (count - os->skip, os->count);
   for (idx = os->skip; num_left > 0 ; idx++) {
     GomResource *resource;
@@ -625,7 +631,7 @@ remove_bookmark (GrlBookmarksSource *bookmarks_source,
 
   g_object_unref (resource);
 
-  if (bookmarks_source->priv->notify_changes) {
+  if (*error != NULL && bookmarks_source->priv->notify_changes) {
     /* We can improve accuracy computing the parent container of removed
        element */
     grl_source_notify_change (GRL_SOURCE (bookmarks_source),
@@ -635,6 +641,31 @@ remove_bookmark (GrlBookmarksSource *bookmarks_source,
   }
 }
 
+static GomResource *
+find_resource (const gchar   *id,
+	       GomRepository *repository)
+{
+  GomResource *resource;
+  GomFilter *filter;
+  GValue value = { 0, };
+
+  if (id == NULL)
+    return NULL;
+
+  g_value_init(&value, G_TYPE_INT64);
+  g_value_set_int64 (&value, g_ascii_strtoll (id, NULL, 0));
+  filter = gom_filter_new_eq (BOOKMARKS_TYPE_RESOURCE, "id", &value);
+  g_value_unset(&value);
+
+  resource = gom_repository_find_one_sync (repository,
+                                           BOOKMARKS_TYPE_RESOURCE,
+                                           filter,
+                                           NULL);
+  g_object_unref (filter);
+
+  return resource;
+}
+
 static void
 store_bookmark (GrlBookmarksSource *bookmarks_source,
                 GList **keylist,
@@ -659,6 +690,7 @@ store_bookmark (GrlBookmarksSource *bookmarks_source,
 
   GRL_DEBUG ("store_bookmark");
 
+  str_id = (gchar *) grl_media_get_id (bookmark);
   title = grl_media_get_title (bookmark);
   url = grl_media_get_url (bookmark);
   thumb = grl_media_get_thumbnail (bookmark);
@@ -684,11 +716,14 @@ store_bookmark (GrlBookmarksSource *bookmarks_source,
     type = BOOKMARK_TYPE_STREAM;
   }
 
-  resource = g_object_new (BOOKMARKS_TYPE_RESOURCE,
-                           "repository", bookmarks_source->priv->repository,
-                           "parent", parent_id,
-                           "type", type,
-                           NULL);
+  resource = find_resource (str_id, bookmarks_source->priv->repository);
+  if (!resource) {
+    resource = g_object_new (BOOKMARKS_TYPE_RESOURCE,
+                             "repository", bookmarks_source->priv->repository,
+                             "parent", parent_id,
+                             "type", type,
+                             NULL);
+  }
 
   if (type == BOOKMARK_TYPE_STREAM) {
     g_object_set (G_OBJECT (resource), "url", url, NULL);
@@ -718,7 +753,7 @@ store_bookmark (GrlBookmarksSource *bookmarks_source,
                               GRLKEYID_TO_POINTER (GRL_METADATA_KEY_DESCRIPTION));
   }
   if (thumb) {
-    g_object_set (G_OBJECT (resource), "thumbnail-url", desc, NULL);
+    g_object_set (G_OBJECT (resource), "thumbnail-url", thumb, NULL);
     *keylist = g_list_remove (*keylist,
                               GRLKEYID_TO_POINTER (GRL_METADATA_KEY_THUMBNAIL));
   }
