@@ -49,8 +49,8 @@ GRL_LOG_DOMAIN_STATIC(local_metadata_log_domain);
 #define TV_REGEX                                \
   "(?<showname>.*)\\."                          \
   "(?<season>[sS\\.]\\d{1,2}|\\d{1,2})"         \
-  "(?<episode>(?:[eExX\\.]\\d{1,2}))"           \
-  "\\.?(?<name>.*|(?:\().*))?"
+  "\\.?(?<episode>(?:(?i)ep|[ex\\.])\\d{1,2})"  \
+  "\\.?(?<name>(?:\().*|.*))?"
 #define MOVIE_REGEX                             \
   "(?<name>.*)"                                 \
   "(?<year>19\\d{2}|20\\d{2})"
@@ -69,17 +69,20 @@ enum {
 
 struct _GrlLocalMetadataSourcePriv {
   gboolean guess_video;
+  GrlKeyID hash_keyid;
 };
 
 /**/
 
 typedef enum {
-  FLAG_VIDEO_TITLE    = 0x1,
-  FLAG_VIDEO_SHOWNAME = 0x2,
-  FLAG_VIDEO_DATE     = 0x4,
-  FLAG_VIDEO_SEASON   = 0x8,
-  FLAG_VIDEO_EPISODE  = 0x10,
-  FLAG_THUMBNAIL      = 0x20,
+  FLAG_VIDEO_TITLE         = 1,
+  FLAG_VIDEO_SHOWNAME      = 1 << 1,
+  FLAG_VIDEO_DATE          = 1 << 2,
+  FLAG_VIDEO_SEASON        = 1 << 3,
+  FLAG_VIDEO_EPISODE       = 1 << 4,
+  FLAG_VIDEO_EPISODE_TITLE = 1 << 5,
+  FLAG_THUMBNAIL           = 1 << 6,
+  FLAG_GIBEST_HASH         = 1 << 7
 } resolution_flags_t;
 
 const gchar *video_blacklisted_prefix[] = {
@@ -87,7 +90,7 @@ const gchar *video_blacklisted_prefix[] = {
 };
 
 const char *video_blacklisted_words[] = {
-  "720p", "1080p",
+  "720p", "1080p", "x264",
   "ws", "WS", "proper", "PROPER",
   "repack", "real.repack",
   "hdtv", "HDTV", "pdtv", "PDTV", "notv", "NOTV",
@@ -120,6 +123,8 @@ static gboolean grl_local_metadata_source_may_resolve (GrlSource *source,
 gboolean grl_local_metadata_source_plugin_init (GrlRegistry *registry,
                                                 GrlPlugin *plugin,
                                                 GList *configs);
+static resolution_flags_t get_resolution_flags (GList                      *keys,
+                                                GrlLocalMetadataSourcePriv *priv);
 
 /**/
 
@@ -241,7 +246,7 @@ static gchar *
 video_sanitise_string (const gchar *str)
 {
   int    i;
-  gchar *line;
+  gchar *line, *line_end;
   GRegex *regex;
 
   line = (gchar *) str;
@@ -253,14 +258,32 @@ video_sanitise_string (const gchar *str)
     }
   }
 
+  /* Get the substring limited by the first blacklisted word */
+  line_end = line + strlen (line);
   for (i = 0; video_blacklisted_words[i]; i++) {
     gchar *end;
 
     end = strcasestr (line, video_blacklisted_words[i]);
-    if (end) {
-      return g_strndup (line, end - line);
+    if (end && end < line_end) {
+      line_end = end;
     }
   }
+
+  if (*line_end != '\0') {
+    line_end = g_utf8_find_prev_char (line, line_end);
+
+    /* After removing substring with blacklisted word, ignore non alpha-numeric
+     * char in the end of the sanitised string */
+    while (g_unichar_isalnum (*line_end) == FALSE &&
+           *line_end != '!' &&
+           *line_end != '?' &&
+           *line_end != '.') {
+      line_end = g_utf8_find_prev_char (line, line_end);
+    }
+
+    return g_strndup (line, line_end - line);
+  }
+
   regex = g_regex_new ("\\.-\\.", 0, 0, NULL);
   line = g_regex_replace_literal(regex, line, -1, 0, ".", 0, NULL);
   g_regex_unref(regex);
@@ -351,6 +374,10 @@ video_guess_values_from_display_name (const gchar *display_name,
       *title = g_match_info_fetch_named (info, "name");
       g_strdelimit (*title, ".()", ' ');
       *title = g_strstrip (*title);
+      if (*title[0] == '\0') {
+        g_free (*title);
+        *title = NULL;
+      }
     }
 
     if (showname) {
@@ -377,7 +404,9 @@ video_guess_values_from_display_name (const gchar *display_name,
     if (episode) {
       gchar *e = g_match_info_fetch_named (info, "episode");
       if (e) {
-        if (*e == 'e' || *e == 'E' || *e == 'x' || *e == '.') {
+        if (g_ascii_strncasecmp (e, "ep", 2) == 0) {
+          *episode = atoi (e + 2);
+        } else if (*e == 'e' || *e == 'E' || *e == 'x' || *e == '.') {
           *episode = atoi (e + 1);
         } else {
           *episode = atoi (e);
@@ -426,6 +455,95 @@ video_guess_values_from_display_name (const gchar *display_name,
   }
 }
 
+static void
+extract_gibest_hash_done (GObject      *source_object,
+                          GAsyncResult *res,
+                          gpointer      user_data)
+{
+  GError *error = NULL;
+  gboolean ret;
+  GrlSourceResolveSpec *rs = user_data;
+
+  ret =  g_task_propagate_boolean (G_TASK (res), &error);
+  if (!ret)
+    rs->callback (rs->source, rs->operation_id, rs->media, rs->user_data, error);
+  else
+    rs->callback (rs->source, rs->operation_id, rs->media, rs->user_data, NULL);
+}
+
+#define CHUNK_N_BYTES (2 << 15)
+
+static void
+extract_gibest_hash (GTask        *task,
+                     gpointer      source_object,
+                     gpointer      task_data,
+                     GCancellable *cancellable)
+{
+  GFile *file = source_object;
+  guint64 buffer[2][CHUNK_N_BYTES/8];
+  GInputStream *stream = NULL;
+  gssize n_bytes, file_size;
+  GError *error = NULL;
+  guint64 hash = 0;
+  gint i;
+  char *str;
+  GrlSourceResolveSpec *rs = task_data;
+  GrlLocalMetadataSourcePriv *priv;
+
+  priv = GRL_LOCAL_METADATA_SOURCE_GET_PRIVATE (g_object_get_data (source_object, "self-source"));
+
+  stream = G_INPUT_STREAM (g_file_read (file, cancellable, &error));
+  if (stream == NULL)
+    goto fail;
+
+  /* Extract start/end chunks of the file */
+  n_bytes = g_input_stream_read (stream, buffer[0], CHUNK_N_BYTES, cancellable, &error);
+  if (n_bytes == -1)
+    goto fail;
+
+  if (!g_seekable_seek (G_SEEKABLE (stream), -CHUNK_N_BYTES, G_SEEK_END, cancellable, &error))
+    goto fail;
+
+  n_bytes = g_input_stream_read (stream, buffer[1], CHUNK_N_BYTES, cancellable, &error);
+  if (n_bytes == -1)
+    goto fail;
+
+  for (i = 0; i < G_N_ELEMENTS (buffer[0]); i++)
+    hash += buffer[0][i] + buffer[1][i];
+
+  file_size = g_seekable_tell (G_SEEKABLE (stream));
+
+  if (file_size < CHUNK_N_BYTES)
+    goto fail;
+
+  /* Include file size */
+  hash += file_size;
+  g_object_unref (stream);
+
+  str = g_strdup_printf ("%" G_GINT64_FORMAT, hash);
+  grl_data_set_string (GRL_DATA (rs->media), priv->hash_keyid, str);
+  g_free (str);
+
+  g_task_return_boolean (task, TRUE);
+  return;
+
+fail:
+  GRL_DEBUG ("Could not get file hash: %s\n", error ? error->message : "Unknown error");
+  g_task_return_error (task, error);
+  g_clear_object (&stream);
+}
+
+static void
+extract_gibest_hash_async (GrlSourceResolveSpec *rs,
+                           GFile                *file,
+                           GCancellable         *cancellable)
+{
+  GTask *task;
+
+  task = g_task_new (G_OBJECT (file), cancellable, extract_gibest_hash_done, rs);
+  g_task_run_in_thread (task, extract_gibest_hash);
+}
+
 static void
 got_file_info (GFile *file,
                GAsyncResult *result,
@@ -436,13 +554,13 @@ got_file_info (GFile *file,
   GError *error = NULL;
   const gchar *thumbnail_path;
   gboolean thumbnail_is_valid;
+  GrlLocalMetadataSourcePriv *priv;
 
   GRL_DEBUG ("got_file_info");
 
-  /* Free stored operation data */
-  cancellable = grl_operation_get_data (rs->operation_id);
+  priv = GRL_LOCAL_METADATA_SOURCE_GET_PRIVATE (g_object_get_data (G_OBJECT (file), "self-source"));
 
-  g_clear_object (&cancellable);
+  cancellable = grl_operation_get_data (rs->operation_id);
 
   info = g_file_query_info_finish (file, result, &error);
   if (error)
@@ -475,7 +593,12 @@ got_file_info (GFile *file,
               grl_media_get_url (rs->media));
   }
 
-  rs->callback (rs->source, rs->operation_id, rs->media, rs->user_data, NULL);
+  if (get_resolution_flags (rs->keys, priv) & FLAG_GIBEST_HASH) {
+    extract_gibest_hash_async (rs, file, cancellable);
+  } else {
+    rs->callback (rs->source, rs->operation_id, rs->media, rs->user_data, NULL);
+    g_clear_object (&cancellable);
+  }
 
   goto exit;
 
@@ -490,6 +613,7 @@ error:
       g_error_free (error);
       g_error_free (new_error);
     }
+    g_clear_object (&cancellable);
 
 exit:
     g_clear_object (&info);
@@ -513,7 +637,8 @@ resolve_video (GrlSource *source,
                  FLAG_VIDEO_SHOWNAME |
                  FLAG_VIDEO_DATE |
                  FLAG_VIDEO_SEASON |
-                 FLAG_VIDEO_EPISODE)))
+                 FLAG_VIDEO_EPISODE |
+                 FLAG_VIDEO_EPISODE_TITLE)))
     return;
 
   if (grl_data_has_key (data, GRL_METADATA_KEY_TITLE)) {
@@ -532,6 +657,8 @@ resolve_video (GrlSource *source,
     0 : FLAG_VIDEO_SEASON;
   miss_flags |= grl_data_has_key (data, GRL_METADATA_KEY_EPISODE) ?
     0 : FLAG_VIDEO_EPISODE;
+  miss_flags |= grl_data_has_key (data, GRL_METADATA_KEY_EPISODE_TITLE) ?
+    0 : FLAG_VIDEO_EPISODE_TITLE;
 
   fill_flags = flags & miss_flags;
 
@@ -559,21 +686,26 @@ resolve_video (GrlSource *source,
              date != NULL ? g_date_time_get_year (date) : 0,
              season, episode);
 
-  /* As this is just a guess, don't erase already provided values,
-   * unless GRL_METADATA_KEY_TITLE_FROM_FILENAME is set */
-  if (grl_data_get_boolean (data, GRL_METADATA_KEY_TITLE_FROM_FILENAME)) {
-    if (fill_flags & FLAG_VIDEO_TITLE) {
-      grl_data_set_string (data, GRL_METADATA_KEY_TITLE, title);
-    }
-    g_free (title);
-  }
-
   if (showname) {
     if (fill_flags & FLAG_VIDEO_SHOWNAME) {
       grl_data_set_string (data, GRL_METADATA_KEY_SHOW, showname);
     }
     g_free (showname);
+
+    if (fill_flags & FLAG_VIDEO_EPISODE_TITLE &&
+        title != NULL) {
+      grl_data_set_string (data, GRL_METADATA_KEY_EPISODE_TITLE, title);
+    }
+  } else {
+    /* As this is just a guess, don't erase already provided values,
+     * unless GRL_METADATA_KEY_TITLE_FROM_FILENAME is set */
+    if (grl_data_get_boolean (data, GRL_METADATA_KEY_TITLE_FROM_FILENAME)) {
+      if (fill_flags & FLAG_VIDEO_TITLE) {
+        grl_data_set_string (data, GRL_METADATA_KEY_TITLE, title);
+      }
+    }
   }
+  g_free (title);
 
   if (date) {
     if (fill_flags & FLAG_VIDEO_DATE) {
@@ -616,6 +748,8 @@ resolve_image (GrlSource *source,
     attributes = G_FILE_ATTRIBUTE_THUMBNAIL_PATH;
 #endif
 
+    g_object_set_data (G_OBJECT (file), "self-source", source);
+
     g_file_query_info_async (file, attributes,
                              G_FILE_QUERY_INFO_NONE, G_PRIORITY_DEFAULT, cancellable,
                              (GAsyncReadyCallback)got_file_info, rs);
@@ -642,7 +776,7 @@ resolve_album_art (GrlSource *source,
   if (!artist || !album)
     return TRUE;
 
-  media_art_get_path (artist, album, "album", NULL, &cache_uri, &thumbnail_uri);
+  media_art_get_path (artist, album, "album", &cache_uri);
 
   if (thumbnail_uri)
     grl_media_set_thumbnail (rs->media, thumbnail_uri);
@@ -717,7 +851,8 @@ has_compatible_media_url (GrlMedia *media)
 }
 
 static resolution_flags_t
-get_resolution_flags (GList *keys)
+get_resolution_flags (GList                      *keys,
+                      GrlLocalMetadataSourcePriv *priv)
 {
   GList *iter = keys;
   resolution_flags_t flags = 0;
@@ -736,6 +871,10 @@ get_resolution_flags (GList *keys)
       flags |= FLAG_VIDEO_EPISODE;
     else if (key == GRL_METADATA_KEY_THUMBNAIL)
       flags |= FLAG_THUMBNAIL;
+    else if (key == priv->hash_keyid)
+      flags |= FLAG_GIBEST_HASH;
+    else if (key == GRL_METADATA_KEY_EPISODE_TITLE)
+      flags |= FLAG_VIDEO_EPISODE_TITLE;
 
     iter = iter->next;
   }
@@ -745,10 +884,23 @@ get_resolution_flags (GList *keys)
 
 /* ================== API Implementation ================ */
 
+static void
+ensure_hash_keyid (GrlLocalMetadataSourcePriv *priv)
+{
+  if (priv->hash_keyid == GRL_METADATA_KEY_INVALID) {
+    GrlRegistry *registry = grl_registry_get_default ();
+    priv->hash_keyid = grl_registry_lookup_metadata_key (registry, "gibest-hash");
+  }
+}
+
 static const GList *
 grl_local_metadata_source_supported_keys (GrlSource *source)
 {
   static GList *keys = NULL;
+  GrlLocalMetadataSourcePriv *priv =
+          GRL_LOCAL_METADATA_SOURCE_GET_PRIVATE (source);
+
+  ensure_hash_keyid (priv);
   if (!keys) {
     keys = grl_metadata_key_list_new (GRL_METADATA_KEY_THUMBNAIL,
                                       GRL_METADATA_KEY_TITLE,
@@ -756,6 +908,8 @@ grl_local_metadata_source_supported_keys (GrlSource *source)
                                       GRL_METADATA_KEY_PUBLICATION_DATE,
                                       GRL_METADATA_KEY_SEASON,
                                       GRL_METADATA_KEY_EPISODE,
+                                      GRL_METADATA_KEY_EPISODE_TITLE,
+                                      priv->hash_keyid,
                                       NULL);
   }
   return keys;
@@ -818,6 +972,7 @@ grl_local_metadata_source_may_resolve (GrlSource *source,
     case GRL_METADATA_KEY_PUBLICATION_DATE:
     case GRL_METADATA_KEY_SEASON:
     case GRL_METADATA_KEY_EPISODE:
+    case GRL_METADATA_KEY_EPISODE_TITLE:
       if (!priv->guess_video)
         return FALSE;
       if (grl_data_has_key (GRL_DATA (media), GRL_METADATA_KEY_URL) &&
@@ -830,6 +985,9 @@ grl_local_metadata_source_may_resolve (GrlSource *source,
       if (grl_data_has_key (GRL_DATA (media), GRL_METADATA_KEY_URL) == FALSE)
         goto missing_url;
       return has_compatible_media_url (media);
+    default:
+      if (key_id == priv->hash_keyid)
+        return has_compatible_media_url (media);
     }
   }
 
@@ -865,7 +1023,7 @@ grl_local_metadata_source_resolve (GrlSource *source,
   /* Can we access the media through gvfs? */
   can_access = has_compatible_media_url (rs->media);
 
-  flags = get_resolution_flags (rs->keys);
+  flags = get_resolution_flags (rs->keys, priv);
   if (grl_data_get_boolean (GRL_DATA (rs->media), GRL_METADATA_KEY_TITLE_FROM_FILENAME))
     flags |= FLAG_VIDEO_TITLE;
 
