@@ -30,6 +30,7 @@
 #include "grl-lua-common.h"
 #include "grl-lua-library.h"
 #include "lua-library/lua-libraries.h"
+#include "lua-library/htmlentity.h"
 
 #define GRL_LOG_DOMAIN_DEFAULT lua_library_log_domain
 GRL_LOG_DOMAIN_STATIC (lua_library_log_domain);
@@ -122,23 +123,15 @@ unescape_string (const char *orig_from)
         } else {
           continue;
         }
-      } else if (strncmp (from, "lt;", 3) == 0) {
-        *to = '<';
-        from += 2;
-      } else if (strncmp (from, "gt;", 3) == 0) {
-        *to = '>';
-        from += 2;
-      } else if (strncmp (from, "amp;", 4) == 0) {
-        *to = '&';
-        from += 3;
-      } else if (strncmp (from, "quot;", 5) == 0) {
-        *to = '"';
-        from += 4;
-      } else if (strncmp (from, "apos;", 5) == 0) {
-        *to = '\'';
-        from += 4;
       } else {
-        continue;
+        gchar *end = NULL;
+
+        end = strstr (from, ";");
+        if (!end)
+          continue;
+
+        *to = html_entity_parse (from, end - from);
+        from = end;
       }
     }
   }
@@ -148,6 +141,31 @@ unescape_string (const char *orig_from)
   return ret;
 }
 
+static void
+grl_data_set_lua_string (GrlData    *data,
+                         GrlKeyID    key_id,
+                         const char *key_name,
+                         const char *str)
+{
+  char *fixed = NULL;
+
+  /* Check for UTF-8 or ISO8859-1 string */
+  if (g_utf8_validate (str, -1, NULL) == FALSE) {
+    fixed = g_convert (str, -1, "UTF-8", "ISO8859-1", NULL, NULL, NULL);
+    if (fixed == NULL) {
+      g_warning ("Ignored non-UTF-8 and non-ISO8859-1 string for field '%s'", key_name);
+      return;
+    }
+  }
+
+  if (fixed) {
+    grl_data_set_string (data, key_id, fixed);
+    g_free (fixed);
+  } else {
+    grl_data_set_string (data, key_id, str);
+  }
+}
+
 /* Top of the stack must be a table */
 static void
 grl_util_add_table_to_media (lua_State *L,
@@ -181,7 +199,7 @@ grl_util_add_table_to_media (lua_State *L,
 
     case G_TYPE_STRING:
       if (lua_isstring (L, -1))
-        grl_data_add_string (GRL_DATA (media), key_id, lua_tostring (L, -1));
+        grl_data_set_lua_string (GRL_DATA (media), key_id, key_name, lua_tostring (L, -1));
       break;
 
     default:
@@ -271,7 +289,7 @@ grl_util_build_media (lua_State *L,
 
       case G_TYPE_STRING:
         if (lua_isstring (L, -1)) {
-          grl_data_set_string (GRL_DATA (media), key_id, lua_tostring (L, -1));
+          grl_data_set_lua_string (GRL_DATA (media), key_id, key_name, lua_tostring (L, -1));
         } else if (lua_istable (L, -1)) {
           grl_util_add_table_to_media (L, media, key_id, key_name, type);
         } else if (!lua_isnil (L, -1)) {
@@ -279,14 +297,28 @@ grl_util_build_media (lua_State *L,
                        lua_typename (L, -1), key_name);
         }
         break;
+      case G_TYPE_BOOLEAN:
+        if (lua_isboolean (L, -1)) {
+          grl_data_set_boolean (GRL_DATA (media), key_id, lua_toboolean (L, -1));
+        } else if (!lua_isnil (L, -1)) {
+          GRL_WARNING ("'%s' is not compatible for '%s'",
+                       lua_typename (L, -1), key_name);
+        }
+        break;
 
       default:
         /* Non-fundamental types don't reduce to ints, so can't be
          * in the switch statement */
         if (type == G_TYPE_DATE_TIME) {
-          GDateTime *date = grl_date_time_from_iso8601 (lua_tostring (L, -1));
-          grl_data_set_boxed (GRL_DATA (media), key_id, date);
-          g_date_time_unref (date);
+          GDateTime *date;
+          const char *date_str = lua_tostring (L, -1);
+          date = grl_date_time_from_iso8601 (date_str);
+          if (date) {
+            grl_data_set_boxed (GRL_DATA (media), key_id, date);
+            g_date_time_unref (date);
+          } else {
+            GRL_WARNING ("'%s' is not a valid ISO-8601 date", date_str);
+          }
         } else if (type == G_TYPE_BYTE_ARRAY) {
            gsize size = luaL_len (L, -1);
            const guint8 *binary = (const guint8 *) lua_tostring (L, -1);
@@ -320,9 +352,10 @@ grl_util_fetch_done (GObject *source_object,
   FetchOperation *fo = (FetchOperation *) user_data;
   lua_State *L = fo->L;
 
-  grl_net_wc_request_finish (GRL_NET_WC (source_object),
-                             res, &data, &len, &err);
-  if (!g_utf8_validate(data, len, NULL)) {
+  if (!grl_net_wc_request_finish (GRL_NET_WC (source_object),
+                                  res, &data, &len, &err)) {
+    data = NULL;
+  } else if (!g_utf8_validate(data, len, NULL)) {
     data = NULL;
     g_set_error_literal (&err, G_IO_ERROR, G_IO_ERROR_INVALID_DATA,
                          "Fetched item is not valid UTF-8");
@@ -618,7 +651,7 @@ grl_l_operation_get_options (lua_State *L)
   }
 
   if (g_strcmp0 (option, "flags") == 0) {
-    GrlResolutionFlags flags = grl_operation_options_get_flags (os->options);
+    GrlResolutionFlags flags = grl_operation_options_get_resolution_flags (os->options);
 
     lua_pushnumber (L, (gint) flags);
     return 1;
@@ -648,6 +681,10 @@ grl_l_operation_get_options (lua_State *L)
       (value) ? (void) lua_pushstring (L, g_value_get_string (value)) : lua_pushnil (L);
       break;
 
+    case G_TYPE_BOOLEAN:
+      (value) ? (void) lua_pushboolean (L, g_value_get_boolean (value)) : lua_pushnil (L);
+      break;
+
     default:
       GRL_DEBUG ("'%s' is being ignored as G_TYPE is not being handled.",
                  key_name);
@@ -789,7 +826,6 @@ grl_l_media_get_keys (lua_State *L)
     }
 
     lua_pushstring (L, key_name);
-    g_free (key_name);
     if (key_id != GRL_METADATA_KEY_INVALID) {
       type = grl_registry_lookup_metadata_key_type (registry, key_id);
       switch (type) {
@@ -802,6 +838,12 @@ grl_l_media_get_keys (lua_State *L)
       case G_TYPE_STRING:
         lua_pushstring (L, grl_data_get_string (GRL_DATA (os->media), key_id));
         break;
+      case G_TYPE_INT64:
+        lua_pushnumber (L, grl_data_get_int64 (GRL_DATA (os->media), key_id));
+        break;
+      case G_TYPE_BOOLEAN:
+        lua_pushboolean (L, grl_data_get_boolean (GRL_DATA (os->media), key_id));
+        break;
       default:
         if (type == G_TYPE_DATE_TIME) {
           GDateTime *date = grl_data_get_boxed (GRL_DATA (os->media), key_id);
@@ -812,12 +854,14 @@ grl_l_media_get_keys (lua_State *L)
         } else {
           GRL_DEBUG ("'%s' is being ignored as G_TYPE is not being handled.",
                      key_name);
+          g_free (key_name);
           lua_pop (L, 1);
           continue;
         }
       }
       lua_settable (L, -3);
     }
+    g_free (key_name);
   }
   g_list_free (list_keys);
   return 1;
