@@ -32,7 +32,8 @@ source = {
   description = "A source for watching Euronews online",
   supported_keys = { "id", "title", "url" },
   supported_media = 'video',
-  tags = { 'news', 'tv' }
+  icon = 'resource:///org/gnome/grilo/plugins/euronews/euronews.svg',
+  tags = { 'news', 'tv', 'net:internet' }
 }
 
 ------------------
