@@ -189,7 +189,6 @@ grl_thetvdb_plugin_init (GrlRegistry *registry,
                          GList *configs)
 {
   GrlTheTVDBSource *source;
-  GParamSpec *spec;
   GrlConfig *config;
   char *api_key = NULL;
 
@@ -207,78 +206,6 @@ grl_thetvdb_plugin_init (GrlRegistry *registry,
     return FALSE;
   }
 
-  spec = g_param_spec_string ("thetvdb-id",
-                              "thetvdb-id",
-                              "TV Show or episode id for The TVDB source.",
-                              NULL,
-                              G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE),
-  GRL_THETVDB_METADATA_KEY_THETVDB_ID =
-    grl_registry_register_metadata_key (registry, spec, NULL);
-  g_param_spec_unref (spec);
-
-  spec = g_param_spec_string ("thetvdb-imdb-id",
-                              "thetvdb-imdb-id",
-                              "TV Show or episode id for IMDB source.",
-                              NULL,
-                              G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE),
-  GRL_THETVDB_METADATA_KEY_IMDB_ID =
-    grl_registry_register_metadata_key (registry, spec, NULL);
-  g_param_spec_unref (spec);
-
-  spec = g_param_spec_string ("thetvdb-zap2it-id",
-                              "thetvdb-zap2it-id",
-                              "TV Show or episode id for Zap2it source.",
-                              NULL,
-                              G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE),
-  GRL_THETVDB_METADATA_KEY_ZAP2IT_ID =
-    grl_registry_register_metadata_key (registry, spec, NULL);
-  g_param_spec_unref (spec);
-
-  spec = g_param_spec_string ("thetvdb-guest-stars",
-                              "thetvdb-guest-stars",
-                              "Guest stars perfoming in the episode.",
-                              NULL,
-                              G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE),
-  GRL_THETVDB_METADATA_KEY_GUEST_STARS =
-    grl_registry_register_metadata_key (registry, spec, NULL);
-  g_param_spec_unref (spec);
-
-  spec = g_param_spec_string ("thetvdb-fanart",
-                              "thetvdb-fanart",
-                              "The mosted voted fanart of the TV Show.",
-                              NULL,
-                              G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE),
-  GRL_THETVDB_METADATA_KEY_FANART =
-    grl_registry_register_metadata_key (registry, spec, NULL);
-  g_param_spec_unref (spec);
-
-  spec = g_param_spec_string ("thetvdb-banner",
-                              "thetvdb-banner",
-                              "The most voted banner of the TV Show.",
-                              NULL,
-                              G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE),
-  GRL_THETVDB_METADATA_KEY_BANNER =
-    grl_registry_register_metadata_key (registry, spec, NULL);
-  g_param_spec_unref (spec);
-
-  spec = g_param_spec_string ("thetvdb-poster",
-                              "thetvdb-poster",
-                              "The most voted poster of the TV Show.",
-                              NULL,
-                              G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE),
-  GRL_THETVDB_METADATA_KEY_POSTER =
-    grl_registry_register_metadata_key (registry, spec, NULL);
-  g_param_spec_unref (spec);
-
-  spec = g_param_spec_string ("thetvdb-episode-screenshot",
-                              "thetvdb-episode-screenshot",
-                              "One screenshot of the episode.",
-                              NULL,
-                              G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE),
-  GRL_THETVDB_METADATA_KEY_EPISODE_SS =
-    grl_registry_register_metadata_key (registry, spec, NULL);
-  g_param_spec_unref (spec);
-
   source = grl_thetvdb_source_new (api_key);
   grl_registry_register_source (registry,
                                 plugin,
@@ -288,7 +215,78 @@ grl_thetvdb_plugin_init (GrlRegistry *registry,
   return TRUE;
 }
 
-GRL_PLUGIN_REGISTER (grl_thetvdb_plugin_init, NULL, SOURCE_ID);
+static void
+grl_thetvdb_plugin_register_keys (GrlRegistry *registry,
+                                  GrlPlugin   *plugin)
+{
+  GParamSpec *spec;
+
+  spec = g_param_spec_string ("thetvdb-id",
+                              "thetvdb-id",
+                              "TV Show or episode id for The TVDB source.",
+                              NULL,
+                              G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE),
+  GRL_THETVDB_METADATA_KEY_THETVDB_ID =
+    grl_registry_register_metadata_key (registry, spec, NULL);
+
+  spec = g_param_spec_string ("thetvdb-imdb-id",
+                              "thetvdb-imdb-id",
+                              "TV Show or episode id for IMDB source.",
+                              NULL,
+                              G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE),
+  GRL_THETVDB_METADATA_KEY_IMDB_ID =
+    grl_registry_register_metadata_key (registry, spec, NULL);
+
+  spec = g_param_spec_string ("thetvdb-zap2it-id",
+                              "thetvdb-zap2it-id",
+                              "TV Show or episode id for Zap2it source.",
+                              NULL,
+                              G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE),
+  GRL_THETVDB_METADATA_KEY_ZAP2IT_ID =
+    grl_registry_register_metadata_key (registry, spec, NULL);
+
+  spec = g_param_spec_string ("thetvdb-guest-stars",
+                              "thetvdb-guest-stars",
+                              "Guest stars perfoming in the episode.",
+                              NULL,
+                              G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE),
+  GRL_THETVDB_METADATA_KEY_GUEST_STARS =
+    grl_registry_register_metadata_key (registry, spec, NULL);
+
+  spec = g_param_spec_string ("thetvdb-fanart",
+                              "thetvdb-fanart",
+                              "The mosted voted fanart of the TV Show.",
+                              NULL,
+                              G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE),
+  GRL_THETVDB_METADATA_KEY_FANART =
+    grl_registry_register_metadata_key (registry, spec, NULL);
+
+  spec = g_param_spec_string ("thetvdb-banner",
+                              "thetvdb-banner",
+                              "The most voted banner of the TV Show.",
+                              NULL,
+                              G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE),
+  GRL_THETVDB_METADATA_KEY_BANNER =
+    grl_registry_register_metadata_key (registry, spec, NULL);
+
+  spec = g_param_spec_string ("thetvdb-poster",
+                              "thetvdb-poster",
+                              "The most voted poster of the TV Show.",
+                              NULL,
+                              G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE),
+  GRL_THETVDB_METADATA_KEY_POSTER =
+    grl_registry_register_metadata_key (registry, spec, NULL);
+
+  spec = g_param_spec_string ("thetvdb-episode-screenshot",
+                              "thetvdb-episode-screenshot",
+                              "One screenshot of the episode.",
+                              NULL,
+                              G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE),
+  GRL_THETVDB_METADATA_KEY_EPISODE_SS =
+    grl_registry_register_metadata_key (registry, spec, NULL);
+}
+
+GRL_PLUGIN_REGISTER_FULL (grl_thetvdb_plugin_init, NULL, grl_thetvdb_plugin_register_keys, SOURCE_ID);
 
 /* ================== TheTVDB GObject ================= */
 
@@ -297,6 +295,10 @@ grl_thetvdb_source_new (const gchar *api_key)
 {
   GObject *object;
   GrlTheTVDBSource *source;
+  const char *tags[] = {
+    "tv",
+    NULL
+  };
 
   GRL_DEBUG ("thetvdb_source_new");
 
@@ -305,6 +307,7 @@ grl_thetvdb_source_new (const gchar *api_key)
                          "source-name", SOURCE_NAME,
                          "source-desc", SOURCE_DESC,
                          "supported-media", GRL_MEDIA_TYPE_VIDEO,
+                         "source-tags", tags,
                          NULL);
 
   source = GRL_THETVDB_SOURCE (object);
@@ -343,11 +346,11 @@ grl_thetvdb_source_init (GrlTheTVDBSource *source)
     grl_metadata_key_list_new (GRL_METADATA_KEY_SEASON,
                                GRL_METADATA_KEY_EPISODE,
                                GRL_METADATA_KEY_GENRE,
-                               GRL_METADATA_KEY_TITLE,
                                GRL_METADATA_KEY_PERFORMER,
                                GRL_METADATA_KEY_DIRECTOR,
                                GRL_METADATA_KEY_PUBLICATION_DATE,
                                GRL_METADATA_KEY_DESCRIPTION,
+                               GRL_METADATA_KEY_EPISODE_TITLE,
                                GRL_THETVDB_METADATA_KEY_THETVDB_ID,
                                GRL_THETVDB_METADATA_KEY_IMDB_ID,
                                GRL_THETVDB_METADATA_KEY_ZAP2IT_ID,
@@ -495,7 +498,7 @@ xml_parse_and_save_episodes (GomRepository *repository,
     GError *error = NULL;
     gint tmp_season_number = -1;
     gint tmp_episode_number = -1;
-    gchar *tmp_title;
+    gchar *tmp_title = NULL;
     gboolean episode_found = FALSE;
     EpisodeResource *eres;
 
@@ -590,7 +593,7 @@ xml_parse_and_save_episodes (GomRepository *repository,
     }
 
     if (title != NULL && tmp_title != NULL
-        && g_ascii_strncasecmp (title, tmp_title, 0) == 0) {
+        && g_ascii_strcasecmp (title, tmp_title) == 0) {
       episode_found = TRUE;
       episode_resource = eres;
     }
@@ -829,12 +832,12 @@ thetvdb_update_media_from_resources (GrlMediaVideo *video,
         failed_keys++;
       break;
 
-    case GRL_METADATA_KEY_TITLE:
+    case GRL_METADATA_KEY_EPISODE_TITLE:
       if (eres != NULL)
         g_object_get (eres, EPISODE_COLUMN_EPISODE_NAME, &str, NULL);
 
       if (str != NULL) {
-        grl_media_set_title (GRL_MEDIA (video), str);
+        grl_media_video_set_episode_title (video, str);
         g_free (str);
       } else
         failed_keys++;
@@ -1529,15 +1532,15 @@ grl_thetvdb_source_may_resolve (GrlSource *source,
 
   /* For season and episode number, we need the the title of the episode */
   if ((key_id == GRL_METADATA_KEY_SEASON || key_id == GRL_METADATA_KEY_EPISODE)
-      && !grl_data_has_key (GRL_DATA (media), GRL_METADATA_KEY_TITLE)) {
+      && !grl_data_has_key (GRL_DATA (media), GRL_METADATA_KEY_EPISODE_TITLE)) {
       if (missing_keys)
-        *missing_keys = grl_metadata_key_list_new (GRL_METADATA_KEY_TITLE, NULL);
+        *missing_keys = grl_metadata_key_list_new (GRL_METADATA_KEY_EPISODE_TITLE, NULL);
 
       return FALSE;
   }
 
   /* For the title of the episode, we need season and episode number */
-  if (key_id == GRL_METADATA_KEY_TITLE) {
+  if (key_id == GRL_METADATA_KEY_EPISODE_TITLE) {
     GList *l = NULL;
     if (!grl_data_has_key (GRL_DATA (media), GRL_METADATA_KEY_SEASON))
         l = g_list_prepend (l, GRLKEYID_TO_POINTER (GRL_METADATA_KEY_SEASON));
@@ -1557,7 +1560,7 @@ grl_thetvdb_source_may_resolve (GrlSource *source,
    * and episode number or the title of the episode */
   if ((key_id == GRL_METADATA_KEY_DIRECTOR
        || key_id == GRL_THETVDB_METADATA_KEY_GUEST_STARS)
-      && !grl_data_has_key (GRL_DATA (media), GRL_METADATA_KEY_TITLE)) {
+      && !grl_data_has_key (GRL_DATA (media), GRL_METADATA_KEY_EPISODE_TITLE)) {
     GList *l = NULL;
     if (!grl_data_has_key (GRL_DATA (media), GRL_METADATA_KEY_SEASON))
         l = g_list_prepend (l, GRLKEYID_TO_POINTER (GRL_METADATA_KEY_SEASON));
