@@ -78,7 +78,7 @@ GRL_LOG_DOMAIN_STATIC(tracker_source_result_log_domain);
   "%s "                                         \
   "?urn tracker:available ?tr . "               \
   "?urn fts:match \"%s\" . "                    \
-  "%s "                                         \
+  "%s %s "                                      \
   "} "                                          \
   "ORDER BY DESC(nfo:fileLastModified(?urn)) "  \
   "OFFSET %u "                                  \
@@ -90,7 +90,7 @@ GRL_LOG_DOMAIN_STATIC(tracker_source_result_log_domain);
   "{ "                                          \
   "%s "                                         \
   "?urn tracker:available ?tr . "               \
-  "%s "                                         \
+  "%s %s "                                      \
   "} "                                          \
   "ORDER BY DESC(nfo:fileLastModified(?urn)) "  \
   "OFFSET %u "                                  \
@@ -107,6 +107,7 @@ GRL_LOG_DOMAIN_STATIC(tracker_source_result_log_domain);
   "?urn nie:isStoredAs ?file . "                \
   "?file tracker:available ?tr . "              \
   "%s "                                         \
+  "%s "                                         \
   "} "                                          \
   "ORDER BY DESC(nfo:fileLastModified(?urn)) "  \
   "OFFSET %u "                                  \
@@ -202,6 +203,39 @@ static GHashTable *grl_tracker_operations;
 
 /**/
 
+static void
+set_title_from_filename (GrlMedia *media)
+{
+  const gchar *url;
+  gchar *path, *display_name, *ext, *title;
+  guint suffix_len;
+
+  url = grl_media_get_url (media);
+  if (url == NULL)
+    return;
+
+  path = g_filename_from_uri (url, NULL, NULL);
+  if (!path)
+    return;
+  display_name = g_filename_display_basename (path);
+  g_free (path);
+  ext = strrchr (display_name, '.');
+  if (!ext)
+    goto out;
+
+  suffix_len = strlen (ext);
+  if (suffix_len != 4 && suffix_len != 5)
+    goto out;
+
+  title = g_strndup (display_name, ext - display_name);
+  if (g_strcmp0 (grl_media_get_title (media), title) == 0)
+    grl_data_set_boolean (GRL_DATA (media), GRL_METADATA_KEY_TITLE_FROM_FILENAME, TRUE);
+  g_free (title);
+
+out:
+  g_free (display_name);
+}
+
 static void
 fill_grilo_media_from_sparql (GrlTrackerSource    *source,
                               GrlMedia            *media,
@@ -213,7 +247,7 @@ fill_grilo_media_from_sparql (GrlTrackerSource    *source,
   tracker_grl_sparql_t *assoc =
     grl_tracker_get_mapping_from_sparql (sparql_key);
   union {
-    gint int_val;
+    gint64 int_val;
     gdouble double_val;
     const gchar *str_val;
   } val;
@@ -265,6 +299,9 @@ fill_grilo_media_from_sparql (GrlTrackerSource    *source,
     } else if (grl_type == G_TYPE_INT) {
       val.int_val = tracker_sparql_cursor_get_integer (cursor, column);
       grl_data_set_int (GRL_DATA (media), grl_key, val.int_val);
+    } else if (grl_type == G_TYPE_INT64) {
+      val.int_val = tracker_sparql_cursor_get_integer (cursor, column);
+      grl_data_set_int64 (GRL_DATA (media), grl_key, val.int_val);
     } else if (grl_type == G_TYPE_FLOAT) {
       val.double_val = tracker_sparql_cursor_get_double (cursor, column);
       grl_data_set_float (GRL_DATA (media), grl_key, (gfloat) val.double_val);
@@ -399,6 +436,7 @@ get_sparql_type_filter (GrlOperationOptions *options,
         fill_grilo_media_from_sparql (GRL_TRACKER_SOURCE (spec->source), \
                                       media, os->cursor, col);          \
       }                                                                 \
+      set_title_from_filename (media);                                  \
                                                                         \
       spec->callback (spec->source,                                     \
                       spec->operation_id,                               \
@@ -504,6 +542,7 @@ tracker_resolve_cb (GObject      *source_object,
       fill_grilo_media_from_sparql (GRL_TRACKER_SOURCE (rs->source),
                                     rs->media, cursor, col);
     }
+    set_title_from_filename (rs->media);
 
     rs->callback (rs->source, rs->operation_id, rs->media, rs->user_data, NULL);
   } else {
@@ -562,6 +601,7 @@ tracker_media_from_uri_cb (GObject      *source_object,
       fill_grilo_media_from_sparql (GRL_TRACKER_SOURCE (mfus->source),
                                     media, cursor, col);
     }
+    set_title_from_filename (media);
 
     mfus->callback (mfus->source, mfus->operation_id, media, mfus->user_data, NULL);
   } else {
@@ -623,6 +663,54 @@ grl_tracker_source_writable_keys (GrlSource *source)
   return keys;
 }
 
+static void
+grl_tracker_source_get_duration_min_max (GrlOperationOptions *options,
+                                         int                 *min,
+                                         int                 *max)
+{
+  GValue *min_val, *max_val;
+
+  grl_operation_options_get_key_range_filter (options, GRL_METADATA_KEY_DURATION,
+                                              &min_val, &max_val);
+  if (min_val)
+    *min = g_value_get_int (min_val);
+  else
+    *min = -1;
+  if (max_val)
+    *max = g_value_get_int (max_val);
+  else
+    *max = -1;
+}
+
+static char *
+grl_tracker_source_create_constraint (int min, int max)
+{
+  if (min <= 0 && max <= 0)
+    return g_strdup ("");
+  if (max <= 0) {
+    return g_strdup_printf ("?urn a nfo:FileDataObject . "
+                            "OPTIONAL {"
+                            "  ?urn nfo:duration ?duration "
+                            "} . "
+                            "FILTER(?duration > %d || !BOUND(?duration))",
+                             min);
+  }
+  if (min <= 0) {
+    return g_strdup_printf ("?urn a nfo:FileDataObject . "
+                            "OPTIONAL {"
+                            "  ?urn nfo:duration ?duration "
+                            "} . "
+                            "FILTER(?duration < %d || !BOUND(?duration))",
+                             max);
+  }
+  return g_strdup_printf ("?urn a nfo:FileDataObject . "
+                          "OPTIONAL {"
+                          "  ?urn nfo:duration ?duration "
+                          "} . "
+                          "FILTER(?duration < %d || ?duration > %d || !BOUND(?duration))",
+                           max, min);
+}
+
 /**
  * Query is a SPARQL query.
  *
@@ -893,22 +981,25 @@ grl_tracker_source_search (GrlSource *source, GrlSourceSearchSpec *ss)
   GrlTrackerOp         *os;
   gint count = grl_operation_options_get_count (ss->options);
   guint skip = grl_operation_options_get_skip (ss->options);
-
+  int min_dur, max_dur;
+  char *duration_constraint;
   GRL_IDEBUG ("%s: id=%u", __FUNCTION__, ss->operation_id);
 
   constraint = grl_tracker_source_get_device_constraint (priv);
   sparql_select = grl_tracker_source_get_select_string (ss->keys);
   sparql_type_filter = get_sparql_type_filter (ss->options, FALSE);
+  grl_tracker_source_get_duration_min_max (ss->options, &min_dur, &max_dur);
+  duration_constraint = grl_tracker_source_create_constraint (min_dur, max_dur);
   if (!ss->text || ss->text[0] == '\0') {
     /* Search all */
     sparql_final = g_strdup_printf (TRACKER_SEARCH_ALL_REQUEST, sparql_select,
-                                    constraint, sparql_type_filter,
+                                    constraint, duration_constraint, sparql_type_filter,
                                     skip, count);
   } else {
     escaped_text = tracker_sparql_escape_string (ss->text);
     sparql_final = g_strdup_printf (TRACKER_SEARCH_REQUEST, sparql_select,
                                     sparql_type_filter, escaped_text,
-                                    constraint, skip, count);
+                                    constraint, duration_constraint, skip, count);
     g_free (escaped_text);
   }
 
@@ -927,6 +1018,7 @@ grl_tracker_source_search (GrlSource *source, GrlSourceSearchSpec *ss)
   g_free (constraint);
   g_free (sparql_select);
   g_free (sparql_type_filter);
+  g_free (duration_constraint);
 }
 
 static void
@@ -944,6 +1036,8 @@ grl_tracker_source_browse_category (GrlSource *source,
   gint count = grl_operation_options_get_count (bs->options);
   guint skip = grl_operation_options_get_skip (bs->options);
   GrlTypeFilter filter = grl_operation_options_get_type_filter (bs->options);
+  int min_dur, max_dur;
+  char *duration_constraint;
 
   GRL_IDEBUG ("%s: id=%u", __FUNCTION__, bs->operation_id);
 
@@ -1032,12 +1126,15 @@ grl_tracker_source_browse_category (GrlSource *source,
                                     grl_metadata_key_tracker_category);
   }
 
+  grl_tracker_source_get_duration_min_max (bs->options, &min_dur, &max_dur);
+  duration_constraint = grl_tracker_source_create_constraint (min_dur, max_dur);
   constraint = grl_tracker_source_get_device_constraint (priv);
   sparql_select = grl_tracker_source_get_select_string (bs->keys);
   sparql_final = g_strdup_printf (TRACKER_BROWSE_CATEGORY_REQUEST,
                                   sparql_select,
                                   category,
                                   constraint,
+                                  duration_constraint,
                                   skip, count);
 
   GRL_IDEBUG ("\tselect: '%s'", sparql_final);
@@ -1054,6 +1151,7 @@ grl_tracker_source_browse_category (GrlSource *source,
 
   g_free (constraint);
   g_free (sparql_select);
+  g_free (duration_constraint);
 }
 
 static void
@@ -1068,12 +1166,16 @@ grl_tracker_source_browse_filesystem (GrlSource *source,
   GrlTrackerOp         *os;
   gint count = grl_operation_options_get_count (bs->options);
   guint skip = grl_operation_options_get_skip (bs->options);
+  int min_dur, max_dur;
+  char *duration_constraint;
 
   GRL_IDEBUG ("%s: id=%u", __FUNCTION__, bs->operation_id);
 
   sparql_select = grl_tracker_source_get_select_string (bs->keys);
   constraint = grl_tracker_source_get_device_constraint (priv);
   sparql_type_filter = get_sparql_type_filter (bs->options, TRUE);
+  grl_tracker_source_get_duration_min_max (bs->options, &min_dur, &max_dur);
+  duration_constraint = grl_tracker_source_create_constraint (min_dur, max_dur);
 
   if (bs->container == NULL ||
       !grl_media_get_id (bs->container)) {
@@ -1109,6 +1211,7 @@ grl_tracker_source_browse_filesystem (GrlSource *source,
   g_free (sparql_type_filter);
   g_free (constraint);
   g_free (sparql_select);
+  g_free (duration_constraint);
 }
 
 void
@@ -1160,22 +1263,9 @@ grl_tracker_source_init_requests (void)
 {
   GrlRegistry *registry = grl_registry_get_default ();
 
-  /* Check if "tracker-category" is registered; if not, the register it */
   grl_metadata_key_tracker_category =
     grl_registry_lookup_metadata_key (registry, "tracker-category");
 
-  if (grl_metadata_key_tracker_category == GRL_METADATA_KEY_INVALID) {
-    grl_metadata_key_tracker_category =
-      grl_registry_register_metadata_key (grl_registry_get_default (),
-                                          g_param_spec_string ("tracker-category",
-                                                               "Tracker category",
-                                                               "Category a media belongs to",
-                                                               NULL,
-                                                               G_PARAM_STATIC_STRINGS |
-                                                               G_PARAM_READWRITE),
-                                          NULL);
-  }
-
   grl_tracker_operations = g_hash_table_new (g_direct_hash, g_direct_equal);
 
   GRL_LOG_DOMAIN_INIT (tracker_source_request_log_domain,
@@ -1191,8 +1281,13 @@ grl_tracker_source_get_caps (GrlSource *source,
   static GrlCaps *caps;
 
   if (!caps) {
+    GList *range_list;
     caps = grl_caps_new ();
     grl_caps_set_type_filter (caps, GRL_TYPE_FILTER_ALL);
+    range_list = grl_metadata_key_list_new (GRL_METADATA_KEY_DURATION,
+                                            GRL_METADATA_KEY_INVALID);
+    grl_caps_set_key_range_filter (caps, range_list);
+    g_list_free (range_list);
   }
 
   return caps;
