@@ -174,38 +174,12 @@ grl_tracker_setup_key_mappings (void)
 {
   GrlRegistry *registry = grl_registry_get_default ();
 
-  /* Check if "tracker-urn" is registered; if not, then register it */
   grl_metadata_key_tracker_urn =
     grl_registry_lookup_metadata_key (registry, "tracker-urn");
 
-  if (grl_metadata_key_tracker_urn == GRL_METADATA_KEY_INVALID) {
-    grl_metadata_key_tracker_urn =
-      grl_registry_register_metadata_key (grl_registry_get_default (),
-                                          g_param_spec_string ("tracker-urn",
-                                                               "Tracker URN",
-                                                               "Universal resource number in Tracker's store",
-                                                               NULL,
-                                                               G_PARAM_STATIC_STRINGS |
-                                                               G_PARAM_READWRITE),
-                                          NULL);
-  }
-
-  /* Check if "gibest-hash" is registered; if not, then register it */
   grl_metadata_key_gibest_hash =
     grl_registry_lookup_metadata_key (registry, "gibest-hash");
 
-  if (grl_metadata_key_gibest_hash == GRL_METADATA_KEY_INVALID) {
-    grl_metadata_key_gibest_hash =
-      grl_registry_register_metadata_key (grl_registry_get_default (),
-                                          g_param_spec_string ("gibest-hash",
-                                                               "Gibest hash",
-                                                               "Gibest hash of the video file",
-                                                               NULL,
-                                                               G_PARAM_STATIC_STRINGS |
-                                                               G_PARAM_READWRITE),
-                                          NULL);
-  }
-
   grl_to_sparql_mapping = g_hash_table_new (g_direct_hash, g_direct_equal);
   sparql_to_grl_mapping = g_hash_table_new (g_str_hash, g_str_equal);
 
@@ -597,6 +571,7 @@ grl_tracker_build_grilo_media (const gchar *rdf_type)
   GrlMedia *media = NULL;
   gchar **rdf_single_type;
   int i;
+  GHashTable *ht;
 
   if (!rdf_type) {
     return NULL;
@@ -605,26 +580,27 @@ grl_tracker_build_grilo_media (const gchar *rdf_type)
   /* As rdf_type can be formed by several types, split them */
   rdf_single_type = g_strsplit (rdf_type, ",", -1);
   i = g_strv_length (rdf_single_type) - 1;
+  ht = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+  for (; i>= 0; i--)
+    g_hash_table_insert (ht, g_path_get_basename (rdf_single_type[i]), GINT_TO_POINTER(TRUE));
 
-  while (!media && i >= 0) {
-    if (g_str_has_suffix (rdf_single_type[i], RDF_TYPE_MUSIC)) {
-      media = grl_media_audio_new ();
-    } else if (g_str_has_suffix (rdf_single_type[i], RDF_TYPE_VIDEO)) {
-      media = grl_media_video_new ();
-    } else if (g_str_has_suffix (rdf_single_type[i], RDF_TYPE_IMAGE)) {
-      media = grl_media_image_new ();
-    } else if (g_str_has_suffix (rdf_single_type[i], RDF_TYPE_ARTIST)) {
-      media = grl_media_box_new ();
-    } else if (g_str_has_suffix (rdf_single_type[i], RDF_TYPE_ALBUM)) {
-      media = grl_media_box_new ();
-    } else if (g_str_has_suffix (rdf_single_type[i], RDF_TYPE_BOX)) {
-      media = grl_media_box_new ();
-    } else if (g_str_has_suffix (rdf_single_type[i], RDF_TYPE_FOLDER)) {
-      media = grl_media_box_new ();
-    }
-    i--;
+  if (g_hash_table_lookup (ht, RDF_TYPE_MUSIC)) {
+    media = grl_media_audio_new ();
+  } else if (g_hash_table_lookup (ht, RDF_TYPE_VIDEO)) {
+    media = grl_media_video_new ();
+  } else if (g_hash_table_lookup (ht, RDF_TYPE_IMAGE)) {
+    media = grl_media_image_new ();
+  } else if (g_hash_table_lookup (ht, RDF_TYPE_ARTIST)) {
+    media = grl_media_box_new ();
+  } else if (g_hash_table_lookup (ht, RDF_TYPE_ALBUM)) {
+    media = grl_media_box_new ();
+  } else if (g_hash_table_lookup (ht, RDF_TYPE_BOX)) {
+    media = grl_media_box_new ();
+  } else if (g_hash_table_lookup (ht, RDF_TYPE_FOLDER)) {
+    media = grl_media_box_new ();
   }
 
+  g_hash_table_destroy (ht);
   g_strfreev (rdf_single_type);
 
   if (!media)
