@@ -61,7 +61,7 @@ GRL_LOG_DOMAIN_STATIC(youtube_log_domain);
 
 #define YOUTUBE_CATEGORIES_ID   "categories"
 #define YOUTUBE_CATEGORIES_NAME N_("Categories")
-#define YOUTUBE_CATEGORIES_URL  "http://gdata.youtube.com/schemas/2007/categories.cat"
+#define YOUTUBE_CATEGORIES_URL  "https://gdata.youtube.com/schemas/2007/categories.cat"
 
 /* ----- Feeds categories ---- */
 
@@ -99,10 +99,10 @@ GRL_LOG_DOMAIN_STATIC(youtube_log_domain);
 
 #define YOUTUBE_MAX_CHUNK       50
 
-#define YOUTUBE_VIDEO_INFO_URL  "http://www.youtube.com/get_video_info?video_id=%s"
-#define YOUTUBE_VIDEO_URL       "http://www.youtube.com/get_video?video_id=%s&t=%s&asv="
-#define YOUTUBE_CATEGORY_URL    "http://gdata.youtube.com/feeds/api/videos/-/%s?&start-index=%s&max-results=%s"
-#define YOUTUBE_WATCH_URL       "http://www.youtube.com/watch?v="
+#define YOUTUBE_VIDEO_INFO_URL  "https://www.youtube.com/get_video_info?video_id=%s"
+#define YOUTUBE_VIDEO_URL       "https://www.youtube.com/get_video?video_id=%s&t=%s&asv="
+#define YOUTUBE_CATEGORY_URL    "https://gdata.youtube.com/feeds/api/videos/-/%s?&start-index=%s&max-results=%s"
+#define YOUTUBE_WATCH_URL       "https://www.youtube.com/watch?v="
 
 #define YOUTUBE_VIDEO_MIME      "application/x-shockwave-flash"
 #define YOUTUBE_SITE_URL        "www.youtube.com"
@@ -320,14 +320,14 @@ grl_youtube_source_new (const gchar *api_key, const gchar *client_id, const gcha
   GDataYouTubeService *service;
   GIcon *icon;
   GFile *file;
+  const char *tags[] = {
+    "net:internet",
+    NULL
+  };
 
   GRL_DEBUG ("grl_youtube_source_new");
 
-#ifdef HAVE_LIBGDATA_0_9
   service = gdata_youtube_service_new (api_key, NULL);
-#else /* HAVE_LIBGDATA_0_9 */
-  service = gdata_youtube_service_new (api_key, client_id);
-#endif /* !HAVE_LIBGDATA_0_9 */
   if (!service) {
     GRL_WARNING ("Failed to initialize gdata service");
     return NULL;
@@ -348,6 +348,7 @@ grl_youtube_source_new (const gchar *api_key, const gchar *client_id, const gcha
                                              "yt-service", service,
                                              "supported-media", GRL_MEDIA_TYPE_VIDEO,
                                              "source-icon", icon,
+                                             "source-tags", tags,
 					     NULL));
 
   g_object_unref (icon);
@@ -444,6 +445,8 @@ release_operation_data (guint operation_id)
   GCancellable *cancellable = grl_operation_get_data (operation_id);
 
   g_clear_object (&cancellable);
+
+  grl_operation_set_data (operation_id, NULL);
 }
 
 static OperationSpec *
@@ -797,6 +800,9 @@ build_media_from_entry_search_cb (GrlMedia *media, gpointer user_data)
 
   if (os->emitted < os->count) {
     remaining = os->count - os->emitted - 1;
+    if (remaining == 0) {
+      release_operation_data (os->operation_id);
+    }
     os->callback (os->source,
 		  os->operation_id,
 		  media,
@@ -1152,7 +1158,6 @@ produce_from_feed (OperationSpec *os)
   query = gdata_query_new_with_limits (NULL , os->skip + 1, os->count);
   os->category_info = &feeds_dir[feed_type];
 
-#ifdef HAVE_LIBGDATA_0_9
   gdata_youtube_service_query_standard_feed_async (GDATA_YOUTUBE_SERVICE (service),
                                                    feed_type,
                                                    query,
@@ -1162,16 +1167,6 @@ produce_from_feed (OperationSpec *os)
                                                    NULL,
                                                    (GAsyncReadyCallback) search_cb,
                                                    os);
-#else /* HAVE_LIBGDATA_0_9 */
-  gdata_youtube_service_query_standard_feed_async (GDATA_YOUTUBE_SERVICE (service),
-                                                   feed_type,
-                                                   query,
-                                                   os->cancellable,
-                                                   search_progress_cb,
-                                                   os,
-                                                   (GAsyncReadyCallback) search_cb,
-                                                   os);
-#endif /* !HAVE_LIBGDATA_0_9 */
 
   g_object_unref (query);
 }
@@ -1214,7 +1209,6 @@ produce_from_category (OperationSpec *os)
   os->category_info = &categories_dir[category_index];
   gdata_query_set_categories (query, category_term);
 
-#ifdef HAVE_LIBGDATA_0_9
   gdata_youtube_service_query_videos_async (GDATA_YOUTUBE_SERVICE (service),
                                             query,
                                             NULL,
@@ -1223,15 +1217,6 @@ produce_from_category (OperationSpec *os)
                                             NULL,
                                             (GAsyncReadyCallback) search_cb,
                                             os);
-#else /* HAVE_LIBGDATA_0_9 */
-  gdata_youtube_service_query_videos_async (GDATA_YOUTUBE_SERVICE (service),
-					    query,
-					    NULL,
-					    search_progress_cb,
-					    os,
-					    (GAsyncReadyCallback) search_cb,
-					    os);
-#endif /* !HAVE_LIBGDATA_0_9 */
 
   g_object_unref (query);
 }
@@ -1409,7 +1394,6 @@ grl_youtube_source_search (GrlSource *source,
   /* Index in GData starts at 1 */
   query = gdata_query_new_with_limits (ss->text, os->skip + 1, os->count);
 
-#ifdef HAVE_LIBGDATA_0_9
   gdata_youtube_service_query_videos_async (GDATA_YOUTUBE_SERVICE (GRL_YOUTUBE_SOURCE (source)->priv->service),
                                             query,
                                             os->cancellable,
@@ -1418,15 +1402,6 @@ grl_youtube_source_search (GrlSource *source,
                                             NULL,
                                             (GAsyncReadyCallback) search_cb,
                                             os);
-#else /* HAVE_LIBGDATA_0_9 */
-  gdata_youtube_service_query_videos_async (GDATA_YOUTUBE_SERVICE (GRL_YOUTUBE_SOURCE (source)->priv->service),
-					    query,
-					    os->cancellable,
-					    search_progress_cb,
-					    os,
-					    (GAsyncReadyCallback) search_cb,
-					    os);
-#endif /* !HAVE_LIBGDATA_0_9 */
 
   g_object_unref (query);
 }
@@ -1452,7 +1427,7 @@ grl_youtube_source_browse (GrlSource *source,
   os->operation_id = bs->operation_id;
   os->container_id = container_id;
   os->keys = bs->keys;
-  os->flags = grl_operation_options_get_flags (bs->options);
+  os->flags = grl_operation_options_get_resolution_flags (bs->options);
   os->skip = grl_operation_options_get_skip (bs->options);
   os->count = grl_operation_options_get_count (bs->options);
   os->callback = bs->callback;
@@ -1562,7 +1537,6 @@ grl_youtube_source_resolve (GrlSource *source,
     grl_operation_set_data (rs->operation_id, cancellable);
     gchar *entryid = g_strconcat ("tag:youtube.com,2008:video:", id, NULL);
 
-#ifdef HAVE_LIBGDATA_0_9
       gdata_service_query_single_entry_async (service,
                                               NULL,
                                               entryid,
@@ -1571,15 +1545,6 @@ grl_youtube_source_resolve (GrlSource *source,
                                               cancellable,
                                               resolve_cb,
                                               rs);
-#else /* HAVE_LIBGDATA_0_9 */
-      gdata_service_query_single_entry_async (service,
-                                              entryid,
-                                              NULL,
-                                              GDATA_TYPE_YOUTUBE_VIDEO,
-                                              cancellable,
-                                              metadata_cb,
-                                              ms);
-#endif /* !HAVE_LIBGDATA_0_9 */
 
       g_free (entryid);
       break;
@@ -1636,7 +1601,6 @@ grl_youtube_get_media_from_uri (GrlSource *source,
   grl_operation_set_data (mfus->operation_id, cancellable);
   entry_id = g_strconcat ("tag:youtube.com,2008:video:", video_id, NULL);
 
-#ifdef HAVE_LIBGDATA_0_9
   gdata_service_query_single_entry_async (service,
                                           NULL,
                                           entry_id,
@@ -1645,15 +1609,6 @@ grl_youtube_get_media_from_uri (GrlSource *source,
                                           cancellable,
                                           media_from_uri_cb,
                                           mfus);
-#else /* HAVE_LIBGDATA_0_9 */
-  gdata_service_query_single_entry_async (service,
-					  entry_id,
-					  NULL,
-					  GDATA_TYPE_YOUTUBE_VIDEO,
-					  cancellable,
-					  media_from_uri_cb,
-					  mfus);
-#endif /* !HAVE_LIBGDATA_0_9 */
 
   g_free (entry_id);
 }
@@ -1662,11 +1617,16 @@ static void
 grl_youtube_source_cancel (GrlSource *source,
                            guint operation_id)
 {
-  GCancellable *cancellable;
+  GCancellable *cancellable = NULL;
+  gpointer data;
 
   GRL_DEBUG (__FUNCTION__);
 
-  cancellable = G_CANCELLABLE (grl_operation_get_data (operation_id));
+  data = grl_operation_get_data (operation_id);
+
+  if (data) {
+    cancellable = G_CANCELLABLE (data);
+  }
 
   if (cancellable) {
     g_cancellable_cancel (cancellable);
