@@ -57,9 +57,10 @@ get_show_for_title (GrlSource  *source,
   keys = grl_metadata_key_list_new (GRL_METADATA_KEY_SHOW,
 				    GRL_METADATA_KEY_SEASON,
 				    GRL_METADATA_KEY_EPISODE,
+				    GRL_METADATA_KEY_EPISODE_TITLE,
 				    NULL);
   options = grl_operation_options_new (NULL);
-  grl_operation_options_set_flags (options, GRL_RESOLVE_FULL);
+  grl_operation_options_set_resolution_flags (options, GRL_RESOLVE_FULL);
 
   grl_source_resolve_sync (source,
 			   media,
@@ -73,7 +74,7 @@ get_show_for_title (GrlSource  *source,
   *season = grl_data_get_int (GRL_DATA (media), GRL_METADATA_KEY_SEASON);
   *episode = grl_data_get_int (GRL_DATA (media), GRL_METADATA_KEY_EPISODE);
   show = g_strdup (grl_data_get_string (GRL_DATA (media), GRL_METADATA_KEY_SHOW));
-  str = grl_media_get_title (media);
+  str = grl_media_video_get_episode_title (GRL_MEDIA_VIDEO (media));
   *new_title = (str && str[0] == '\0') ? NULL : g_strdup (str);
 
   g_object_unref (media);
@@ -101,6 +102,7 @@ test_episodes (void)
     { "Boardwalk.Empire.S04E01.HDTV.x264-2HD.mp4", NULL, "Boardwalk Empire", NULL, 4, 1 },
     { NULL, "file:///home/test/My%20super%20series.S01E01.mp4", "My super series", NULL, 1, 1 },
     { "Adventure Time - 2x01 - It Came from the Nightosphere.mp4", NULL, "Adventure Time", "It Came from the Nightosphere", 2, 1 },
+    { "Real Humans S01 EP01 [X264] [HD 720p] [FR] [SWE] [SRT FR] [MRPHU].mkv", NULL, "Real Humans", NULL, 1, 1 },
 
     /* Episode and Series separated by '.' and Title inside parenthesis */
     { NULL, "file:///home/toso/Downloads/Felicity/Felicity%202.05%20(Crash).avi", "Felicity", "Crash", 2, 5 },
@@ -164,7 +166,7 @@ test_title_override (void)
 
     keys = grl_metadata_key_list_new (GRL_METADATA_KEY_TITLE, GRL_METADATA_KEY_SHOW, NULL);
     options = grl_operation_options_new (NULL);
-    grl_operation_options_set_flags (options, GRL_RESOLVE_FULL);
+    grl_operation_options_set_resolution_flags (options, GRL_RESOLVE_FULL);
 
     grl_source_resolve_sync (source,
 			     media,
